import nl.saxion.app.CsvReader;
import nl.saxion.app.SaxionApp;

import java.awt.*;
import java.util.ArrayList;

public class Application implements Runnable {

    public static void main(String[] args) {
        SaxionApp.start(new Application(), 800, 768);
    }

    ArrayList<Passenger> passengers = new ArrayList<>();

    public void run() {
        //Start here! Good luck with the test!
        int input = -1;
        while(input != 0) {
            SaxionApp.clear();
            displayMenu();
            input = getUserIntBetween(0,4, "Menu option does not exist");
            if(input == 1) {
                readPassengerCSV();
            } else if (input == 2) {
                showAllPassengers();
                SaxionApp.pause();
            } else if(input ==3) {
                addPassenger();
                SaxionApp.printLine("Passenger successfully added.");
                SaxionApp.pause();
            } else if(input == 4) {
                showStatistics();
                SaxionApp.pause();
            }
        }
    }

    public void showStatistics() {
        SaxionApp.clear();
        SaxionApp.drawText("The Passenger List", 0, 0, 32);
        SaxionApp.drawText("The oldest passenger is " + getOldestPassenger().name, 0, 40, 20);
        double totalWeight = calculateTotalPassengerWeight();
        SaxionApp.drawText("The total weight of all the passengers is " + String.format("%.1f", totalWeight), 0, 60, 20);
        SaxionApp.drawText("Percentage of passengers per class ", 0, 90, 20);
        drawGraph();

    }

    public void drawGraph() {
        SaxionApp.drawText("Class 1", 0, 120, 15);
        SaxionApp.drawText("Class 2", 0, 150, 15);
        SaxionApp.drawText("Class 3", 0, 180, 15);
        SaxionApp.turnBorderOff();
        SaxionApp.setFill(Color.DARK_GRAY);
        SaxionApp.drawRectangle(200, 120, 500, 20);
        SaxionApp.drawRectangle(200, 150, 500, 20);
        SaxionApp.drawRectangle(200, 180, 500, 20);
        SaxionApp.setFill(Color.YELLOW);
        SaxionApp.drawRectangle(200, 120, 500*getClassCount(1)/passengers.size(), 20);
        SaxionApp.drawRectangle(200, 150, 500*getClassCount(2)/passengers.size(), 20);
        SaxionApp.drawRectangle(200, 180, 500*getClassCount(3)/passengers.size(), 20);
    }

    public int getClassCount(int classNumber) {
        int classTotal = 0;
        for(Passenger passenger: passengers) {
            if(passenger.classNumber == classNumber) {
                classTotal++;
            }
        }
        return classTotal;
    }

    public Passenger getOldestPassenger() {
        Passenger oldest = passengers.get(0);
        for(Passenger passenger: passengers) {
            if(passenger.age > oldest.age) {
                oldest = passenger;
            }
        }
        return oldest;
    }

    public double calculateTotalPassengerWeight() {
        double totalWeight = 0.0;
        for(Passenger passenger: passengers) {
            totalWeight += passenger.weight;
        }
        return totalWeight;
    }

    public int getUserIntBetween(int min, int max, String errorMessage) {
        int input = SaxionApp.readInt();
        while(input < min || input > max) {
            SaxionApp.printLine(errorMessage, Color.red);
            input = SaxionApp.readInt();
        }
        return input;
    }

    public double getUserDoubleBetween(double min, double max, String errorMessage) {
        double input = SaxionApp.readDouble();
        while(input < min || input > max) {
            SaxionApp.printLine(errorMessage, Color.red);
            input = SaxionApp.readDouble();
        }
        return input;
    }

    public void showAllPassengers() {
        for(Passenger p: passengers) {
            SaxionApp.printLine(formatPassenger(p));
        }
    }

    public String formatPassenger(Passenger p) {
        return p.name + " (" + p.sex + ")"
                + " age:" + p.age
                + " weight:" + p.weight
                + " travels in class:" + p.classNumber;
    }

    public void addPassenger() {
        Passenger newPassenger = new Passenger();
        SaxionApp.printLine("What is the name of the passenger?");
        newPassenger.name = SaxionApp.readString();
        while(newPassenger.name.length() == 0) {  // or use isEmpty()
            SaxionApp.printLine("Name length can't be 0");
            newPassenger.name = SaxionApp.readString();
        }
        SaxionApp.printLine("What is the sex of the passenger (M, F, N)?");
        newPassenger.sex = SaxionApp.readChar(); // String would also be fine.
        while(!(newPassenger.sex == 'M' || newPassenger.sex == 'F' || newPassenger.sex == 'N')) {
            SaxionApp.printLine("Incorrect format for passenger sex", Color.red);
            SaxionApp.printLine("What is the sex of the passenger (M, F, N)?");
            newPassenger.sex = SaxionApp.readChar();
        }
        SaxionApp.printLine("What is the class of the passenger?");
        newPassenger.classNumber = getUserIntBetween(1, 3, "Incorrect format for passenger class");
        SaxionApp.printLine("What is the age of the passenger?");
        newPassenger.age = getUserIntBetween(0, 110, "Incorrect age for passenger.");
        SaxionApp.printLine("What is the weight of the passenger?");
        newPassenger.weight = getUserDoubleBetween(40.0, 120.0, "Incorrect weight for the passenger");
        passengers.add(newPassenger);
    }

    public void displayMenu() {
        SaxionApp.printLine("Welcome to the Britania's Passengerlist!");
        SaxionApp.printLine("------------------------");
        SaxionApp.printLine("Number of passengers: " + passengers.size());
        SaxionApp.printLine("1) Read from passengers.csv");
        SaxionApp.printLine("2) Show all passengers");
        SaxionApp.printLine("3) Add a passenger");
        SaxionApp.printLine("4) Show statistics");
        SaxionApp.printLine("0) Exit");

    }

    public void readPassengerCSV() {
        CsvReader csvReader = new CsvReader("passengers.csv");
        csvReader.setSeparator(',');
        csvReader.skipRow();

        while(csvReader.loadRow()) {
            Passenger newPassenger = new Passenger();
            newPassenger.name = csvReader.getString(0);
            newPassenger.sex = csvReader.getString(1).charAt(0); //String would also be fine.
            newPassenger.classNumber = csvReader.getInt(2);
            newPassenger.weight = csvReader.getDouble(3);
            newPassenger.age = csvReader.getInt(4);
            passengers.add(newPassenger);
        }

    }
}






